#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmail.com>'
__copyright__ = "Copyright (C) 2012 PYMETRICK.ORG"
__date__ = '2012-09-21'
__version__ = '0.4'
__license__ = 'Licencia GNU/GPLv2'
__credits__ = ''
__text__ = 'Tratamiento fiscal, bancos y monedas'
__file__ = 'exchequer.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version


import sys, os, imp
import string
import re
import urllib

# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;

def validate_cif(value):
    value = unicode(value).upper()
    match = re.match('(?P<letra>[A-H]|[J-N]|[P-S]|[UVW])(?P<numeracion>\d{7})(?P<control>[0-9A-J])', value)
    if match is None:
        match = re.match('(?P<letra>[A-H]|[J-N]|[P-S]|[UVW])-(?P<numeracion>\d{7})(?P<control>[0-9A-J])', value)
    if match is None:
        raise Expection
    LETRAS_CIF = 'ABCDEFGHJKLMNPQRSUVW' # Letras de CIF válidas
    DC_NUMERICO_CIF = 'ABEH' # CIFs con número como dígito de control
    DC_LETRA_CIF = 'KPQS' # CIFs con letra como dígito de control
    ALPHA_CIF = 'JABCDEFGHI' # Correspondencia entre letra y dígito de control
    # Obtenemos la información
    letra = match.group('letra')
    numeracion = match.group('numeracion')
    control = match.group('control')
    # No todas las letras son válidas
    if letra not in LETRAS_CIF:
        raise Expection
    # Suma de los pares
    a = reduce(lambda x, y: int(x)+int(y), numeracion[1::2])
    # Suma de la suma de los dígitos de los impares * 2
    b = sum(
        [
            int(reduce(lambda x,y: int(x)+int(y), impar))
            for impar in map(lambda x: unicode(int(x)*2), numeracion[::2])
        ]
    )
    c = a + b
    e = c % 10
    d = (10-e) % 10
    # Obtenemos el dígito de control
    if letra in DC_NUMERICO_CIF:
        d = unicode(d) # Es numérico
    elif letra in DC_LETRA_CIF:
        d = ALPHA_CIF[d] # Es una letra
    else:
        # Puede ser tanto letra como número
        if control.isalpha():
            d = ALPHA_CIF[d]
        else:
            d = unicode(d)
    if d != control:
        raise Expection
    return unicode('%s%s%s' % (letra, numeracion, control))

def validateNIF(value):
    value = unicode(value).upper()
    match = re.match('(?P<dni>\d{8})(?P<letra>[A-Z])', value)
    if match is None:
        match = re.match('(?P<dni>\d{8})-(?P<letra>[A-Z])', value)
    if match is None:
        raise Expection

    dni = int(match.group('dni'))
    letra = match.group('letra')

    LETRAS_NIF = 'TRWAGMYFPDXBNJZSQVHLCKE'
    if LETRAS_NIF[dni%23] != letra:
        raise Expection
    return unicode('%s%s' % (dni, letra))

def validate_NIF(value):
    value = value.upper().replace(' ','').replace('-','')
    number = int(float(re.findall(r'[\d]{1,8}',value)[0]))
    letter = re.findall(r'[A-Z]{1,8}',value)
    LETRAS_NIF = 'TRWAGMYFPDXBNJZSQVHLCKE'
    if LETRAS_NIF[number%23] != letter[len(letter)-1]:
        return value
    else:
        return ''


def currencyExchangeEUR(dest=None):
    """Obtiene lista diaria de cambios desde BCE en formato XML
    y extrae informacion de cambio de monedas a partir de 1 EUR
    """
    currencyList = list()
    currencyDict = dict()
    url = "http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml"
    currency = urllib.urlopen(url).read()
    buscar = r"currency\s\'[\w{3}]\s\'?rate=\'(\d{4,7}.)\'/>"
    for m in re.findall(r"currency='[\w]{3}'\srate='[\d]{1,5}[.]?[\d]{1,5}'",currency):
        currencyList = list(m.replace('currency=','').replace(' rate','').replace("'",'').split('='))
        currencyDict[currencyList[0]]=float(currencyList[1])
    if dest is not None and dest in currencyDict:
        return currencyDict[dest]
    else:
        return 0





#esto es un test
if __name__ == '__main__' :
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help\n")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print u'''Tratamiento fiscal, bancario y monedas'''
        print u'''Devuelve el cambio diario de una moneda a partir de 1 EURO segun BCE currencyExchange(dest=None)\n'''
    print currencyExchangeEUR('PLN')
