#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmail.com>'
__copyright__ = "Copyright (C) 2012 PYMETRICK.ORG"
__date__ = '2012-09-21'
__version__ = '0.4'
__license__ = 'Licencia GNU/GPLv2'
__credits__ = ''
__text__ = 'Utilidades diversas'
__file__ = 'helpers.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version

import sys
import os
import imp
import string
import re
import unicodedata
import ConfigParser
import hashlib
import locale
import logging
import codecs
import urllib
import itertools       # combinations y permutations
from datetime import datetime
from time import localtime,mktime
from decimal import Decimal
from random import sample, choice
try:
    import simplejson as json
except:
    import json
from common import *

# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;

if PY2:
    # python3 chr & python2 unichr
    chr = unichr
    
if PY3:
    long = int

# Gestion de TIMESTAMP #########################################################################################

# Tratamiento de hora para producir timestamp y su conversion desde timestamp a hora normalizada
def datetime2timestamp(__datetime__=None):
    '''Realiza conversion de 2014-02-04 12:21:27 a 1391512887'''
    t = 0
    if __datetime__ is None:
        # Si no se le indica una fecha como parametro, obtendra la fecha del sistema
        __datetime__ = datetime.now()
    if type(__datetime__).__name__=='datetime':
        t = int(mktime(__datetime__.timetuple()))
    return t

def timestamp2datetime(__timestamp__=None):
    '''Realiza conversion de 1391512887 a 2014-02-04 12:21:27'''
    t = ''
    if __timestamp__ is not None:
        if type(__timestamp__).__name__=='int' or type(__timestamp__).__name__=='float':
            t = datetime.fromtimestamp(int(__timestamp__))
    return t

cadena = ["0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"]

def encodeDateTime2String(isValue):
    """codificar un formato date-time yyyymmddsssss a string en base 36 segun list 'cadena'
       @parameters
           isValue con formato string p.ejemplo: 2016031736936 -> PQ5H8RXK
       @return
           devuelve un string en base 36 p.ejemplo PQ5H8RXK
    """    
    stringResult = ""
    nTotal = 0
    if type(isValue).__name__ == 'datetime':
        _decimal_ = int(datetime.strftime(isValue,"%Y%m%d"))*100000 + int(datetime.strftime(isValue,"%H"))*3600 + int(datetime.strftime(isValue,"%M"))*60 + int(datetime.strftime(isValue,"%S"))
    else:    
        _decimal_ = int(isValue)
    while _decimal_>0:
        if _decimal_>36:
            stringResult = cadena[_decimal_%36] + stringResult
            _decimal_ = int(_decimal_/36)
        else:
            stringResult = cadena[_decimal_] + stringResult
            _decimal_ = 0
    return stringResult

def decodeString2DateTime(isString,dtime=None):
    """decodificar un formato string en base 36 a date-time yyyymmddsssss segun list 'cadena'
       @parameters
           isString con formato string p.ejemplo: PQ5H8RXK -> 2016031736936
           dtime si True responde con un string en formato datetime
       @return
           dtime=None responde con un string en formato yyyymmddsssss
           dtime=True responde con un objeto datetime yyyy-mm-dd hh:mm:ss.sssss
    """
    nResult = 0
    for n in range(len(isString)):
        nBase = 36
        h = 0
        for m in range(len(isString)-2-n):
            h = h + 1
            nBase = nBase*36
        if n+1<len(isString):
            nResult = nResult+((long(cadena.index(isString[n])))*nBase)
        else:
            nResult = nResult+((long(cadena.index(isString[n]))))
    stringResult = str(nResult)
    if dtime:    
        y = stringResult[0:4]
        _y = int(y)
        m = stringResult[4:6]
        _m = int(m)
        d = stringResult[6:8]
        _d = int(d)
        tt = stringResult[-5:]
        ss = int(tt)
        hh = int(ss/3600)
        ss = ss - (hh*3600)
        mm = int(ss/60)
        ss = ss - (mm*60)
        return datetime(_y,_m,_d,hh,mm,ss)
    else:
        return stringResult

# Gestion de REGION ###########################################################################################

# Formato de fecha local para un pais
def localeES():
    # asignacion locale española
    d = datetime.datetime(2000,12,30)
    locale.setlocale(locale.LC_TIME, "sp")
    # 'Spanish_Spain.1252'
    fecha.strftime("%x")
    #'30/12/2000'

    # locale ingles
    locale.setlocale(locale.LC_TIME, "en")
    #'English_United States.1252'
    fecha.strftime("%x")
    # '12/30/2000'

    print (time.localtime())
    # time.struct_time(tm_year=2014, tm_mon=2, tm_mday=4, tm_hour=20, tm_min=1,
    # tm_sec=23, tm_wday=1, tm_yday=35, tm_isdst=0)

# Gestion de FICHEROS CONFIGURACION ###########################################################################

def read_config(file,section,title):
    """Leer datos de ficheros de configuracion
    <section> se identifica en fichero como [...]
    y contenido en la <section> se encuentra <title...  :>
    """
    config = ConfigParser.ConfigParser()
    config.read(file)
    Value = config.get(section,title)
    return Value

def save_config(file,section,title,value):
    """Guardar o modificar datos en fichero de configuracion
    <section> se identifica en fichero como [...]
    y contenido en la <section> se encuentra <title    ...  :>
    """
    config = ConfigParser.ConfigParser()
    config.add_section(section)
    config.set(section,title,value)
    # write to screen
    config.write(sys.stdout)
    f = open(file, 'w')
    config.write(f)
    f.close()
    return ''

# Gestion de TEXTOS ###########################################################################################

def get_keywords (text=None, exclusion_list=list()):
    """Devuelve una lista de palabras de un texto ordenado por el numero de apariciones
       permite una lista de palabras a excluir del analisis
    """
    keywords = []
    list_of_words = re.split('\W+', text )

    for exclusion_word in exclusion_list:
        try:
            list_of_words.remove (exclusion_word)
        except ValueError:
            pass
    words_count = []
    while list_of_words != []:
        word = list_of_words[0]
        words_count.append( (list_of_words.count( word ), word ) )
        salir = False
        while (not salir):
            try:
                list_of_words.remove( word )
            except (ValueError, IndexError):
                salir = True

    words_count.sort( key=lambda word_count : word_count[0], reverse = True )

    for word in words_count:
        keywords.append( word[1] )

    return keywords

def do_wordcount(text,word):
    """Cuantas veces una palabra aparece en un texto"""
    if text.__len__() and word.__len__():
        return len(re.findall(word, text))
    else:
        return []

def strings_differ(string1, string2):
    """Check whether two strings differ while avoiding timing attacks.

    This function returns True if the given strings differ and False
    if they are equal.  It's careful not to leak information about *where*
    they differ as a result of its running time, which can be very important
    to avoid certain timing-related crypto attacks:

        http://seb.dbzteam.org/crypto/python-oauth-timing-hmac.pdf

    """
    if len(string1) != len(string2):
        return True

    invalid_bits = 0
    for a, b in zip(string1, string2):
        invalid_bits += a != b

    return invalid_bits != 0

def latin2html(latin_text):
    """ Sustituye caracteres especiales por codificacion html"""
    nString = list()
    cString = ''
    latin_text = string2unicode(latin_text)
    for n in latin_text:
        if LATIN_DICT.get(ord(n),None):
            nString.append(LATIN_DICT[ord(n)])
        else:
            nString.append(n)
    # Elimina todos los caracteres por encima de 128 caracteres ASCII que no han podido transformarse a HTML
    cString = unicodedata.normalize('NFKD', u''.join(nString)).encode('ascii','ignore')
    # devuelve string en formato html
    return cString

def string2unicode(cString):
    """ Intenta aplicar unicode a una cadena de caracteres indicando que se trata de utf-8 """
    try:
        cString = unicode(cString, 'utf-8')
    except TypeError:
        pass
    return cString


def str2unicode(_str):
    """ Intenta aplicar unicode a una cadena de caracteres indicando que se trata de utf-8 """
    _type = None
    # codificacion por defecto
    _encoding = sys.getdefaultencoding()
    # comprobar si es dict o list o tuple
    if type(_str).__name__ == 'dict' or type(_str).__name__=='list' or type(_str).__name__=='tuple':
        _type = type(_str).__name__
        _str = json.dumps(_str)
   
    # decodifica cadenas utf-8 url
    _str = urllib.unquote(urllib.unquote(_str))
    _bytes = None
    if not isinstance(_str,unicode):
        encodings = ['ascii','utf_8']
        for e in encodings:
            try:
                _bytes = unicode(_str,e)
            except:
                pass
            else:
                if not isinstance(_bytes,unicode):
                    _bytes = None
                break
    else:
        # devolver el unicode recibido
        _bytes = _str
    if _type:
        return json.loads(_bytes)
    else:
        return _bytes

def do_truncate(s, length=255, killwords=False, end='...'):
    """Return a truncated copy of the string. The length is specified
       with the first parameter which defaults to ``255``. If the second
       parameter is ``true`` the filter will cut the text at length. Otherwise
       it will try to save the last word. If the text was in fact
       truncated it will append an ellipsis sign (``"..."``). If you want a
       different ellipsis sign than ``"..."`` you can specify it using the
       third parameter.

       {{ mytext|truncate(300, false, '&raquo;') }}
       truncate mytext to 300 chars, don't split up words, use a
       right pointing double arrow as ellipsis sign.
    """
    if len(s) <= length:
        return s
    elif killwords:
        return s[:length] + end
    words = s.split(' ')
    result = []
    m = 0
    for word in words:
        m += len(word) + 1
        if m > length:
            break
        result.append(word)
    result.append(end)
    return u' '.join(result)

def stem(words=None,language='ES'):
    ''' A partir de una lista de palabras, las transforma en su plural o singular (ES) y
    se obtenemos una list con sus diferentes combinaciones, para realizar busquedas 
    '''
    try:
        if words is not None and language == 'ES':
            if type(words).__name__ == 'list':
                words = map(lambda x:x.lower(),words)
                # buscar por nombre de producto
                # comprobar si palabra plural -> singular o singular->plural
                list_stemming = list()
                words_search = list()
                for n in words:
                    if n.endswith('es'):
                        if n[:-2].endswith('c'):
                            list_stemming.append(''.join([n[:-3],'z']))
                        else:
                            list_stemming.append(n[:-2])
                    elif n.endswith('s'):
                        list_stemming.append(n[:-1])
                    elif n.endswith('z'):
                        list_stemming.append(''.join([n[:-1],'ces']))                                
                    elif n.endswith(('&iacute;','&uacute;')):
                        list_stemming.append(''.join([n,'es']))
                    elif n.endswith(('a','e','i','o','u')):
                        list_stemming.append(''.join([n,'s']))
                    else:
                        list_stemming.append(''.join([n,'es']))
                if len(words)>1:
                    # para multiples palabras
                    # permutaciones de elementos, hasta un maximo de 4 palabras
                    for n in itertools.permutations(words[:4]):
                        w = list()
                        w.append(n[0])
                        for m in range(1,len(n)):
                            w.append('.+%s' % n[m])
                        words_search.append(''.join(w))   # se guardan permutaciones de la forma palabra1.+palabra2.+palabra3.+palabra4
                    # permutaciones de elementos, hasta un maximo de 4 list_stemming
                    for n in itertools.permutations(list_stemming[:4]):
                        w = list()
                        w.append(n[0])
                        for m in range(1,len(n)):
                            w.append('.+%s' % n[m])
                        words_search.append(''.join(w))   # se guardan permutaciones de la forma palabra1.+palabra2.+palabra3.+palabra4
                else:
                    #palabras sencillas
                    words_search.append(words[0])
                    if len(list_stemming)>0:
                        words_search.append(list_stemming[0])
                return words_search
        return []
    except TypeError:
        pass
    
# Gestion de FICHEROS #########################################################################################

def import_from(uri,absl=False):
    """Importar un modulo dinamicamente"""
    if not absl:
        uri= os.path.normpath(os.path.join(os.path.dirname(__file__),uri))

    path, fname = os.path.split(uri)

    mname, ext = os.path.splitext(fname)

    no_ext = os.path.join(path, mname)
    if os.path.exists(no_ext + '.pyc'):
        try:
            return imp.load_compiled(mname, no_ext + '.pyc')
        except:
            pass
    if os.path.exists(no_ext + '.py'):
        try:
            return imp.load_source(mname, no_ext + '.py')
        except:
            pass

def csv2list(_path_,separator=','):
    """Como parametro se le indica un fichero csv y devuelve su contenido como lista"""
    try:
        datafile = open(_path_, 'r')
        data = []
        for row in datafile:
            data.append(row.strip().split(separator))
        datafile.close()
        return data
    except ValueError:
        return None

def purge(fn, pattern):
    """Delete files in a dir matching pattern
    """
    for f in os.listdir(fn):
        if re.search(pattern, f):
            os.remove(os.path.join(fn, f))


# Gestion de OBJETOS ##########################################################################################

def sort_dict(d):
    """Ordena claves de un dict"""
    keys = d.keys()
    keys.sort()
    return "{" + ", ".join(["%r: %r" % (k, d[k]) for k in keys]) + "}"

def remove_empty(d):
    """Elimina los valores nulos de un list o dict"""
    if d:
        empty_check = True   # valor inicial de comprobacion
    else:
        empty_check = False
    while empty_check:
        empty_check = False
        if type(d).__name__ == 'dict':
            for k in d.keys():
                try:
                    if not d[k]:
                        del d[k]
                        # solicitar nueva comprobacion
                        empty_check = True
                    else:
                        if type(d[k]).__name__ == 'dict' or type(d[k]).__name__ == 'list':
                            d[k] = remove_empty(d[k])
                            if not d[k]:
                                # solicitar nueva comprobacion
                                empty_check = True
                except ValueError:
                    pass
        elif type(d).__name__ == 'list':
            for k in d:
                try:
                    if not k:
                        d.remove(k)
                        # solicitar nueva comprobacion
                        empty_check = True
                    else:
                        if type(k).__name__ == 'dict' or type(k).__name__ == 'list':
                            k = remove_empty(k)
                            if not k:
                                # solicitar nueva comprobacion
                                empty_check = True
                except ValueError:
                    pass
    return d

def cartesian_product(x_list = None):
    """ Producto cartesiano de list hasta un maximo de 3 """
    try:
        basestring
    except NameError:
        basestring = str
        
    if x_list is not None:
        if type(x_list).__name__ == 'list':
            values = { 
                       1: "[[frst] for frst in x_list[0]]", 
                       2: "[[frst,scnd] for frst in x_list[0] for  scnd in x_list[1]]", 
                       3: "[[frst,scnd,thrth] for  frst in x_list[0] for scnd in x_list[1] for  thrth in x_list[2]]"}
             
            listcomp = eval(values.get(len(x_list)))
            for n in range(len(listcomp)):
                for m in range(len(listcomp[n])):
                    if not isinstance(listcomp[n][m], (basestring)):
                        listcomp[n][m] = codecs.encode(str(listcomp[n][m]), 'utf-8')
            return listcomp
    return list()


def object_description(object):
    """ Produce a human-consumable text description of ``object``,
        usually involving a Python dotted name. For example:

       >>> object_description(None)
       u'None'
       >>> from xml.dom import minidom
       >>> object_description(minidom)
       u'module xml.dom.minidom'
       >>> object_description(minidom.Attr)
       u'class xml.dom.minidom.Attr'
       >>> object_description(minidom.Attr.appendChild)
       u'method appendChild of class xml.dom.minidom.Attr'
       >>>

       If this method cannot identify the type of the object, a generic
       description ala ``object <object.__name__>`` will be returned.

       If the object passed is already a string, it is simply returned.  If it
       is a boolean, an integer, a list, a tuple, a set, or ``None``, a
       (possibly shortened) string representation is returned.
    """
    if isinstance(object, string_types):
        return text_(object)
    if isinstance(object, integer_types):
        return text_(str(object))
    if isinstance(object, (bool, float, type(None))):
        return text_(str(object))
    if isinstance(object, set):
        if PY3: # pragma: no cover
            return shortrepr(object, '}')
        else:
            return shortrepr(object, ')')
    if isinstance(object, tuple):
        return shortrepr(object, ')')
    if isinstance(object, list):
        return shortrepr(object, ']')
    if isinstance(object, dict):
        return shortrepr(object, '}')
    module = inspect.getmodule(object)
    if module is None:
        return text_('object %s' % str(object))
    modulename = module.__name__
    if inspect.ismodule(object):
        return text_('module %s' % modulename)
    if inspect.ismethod(object):
        oself = getattr(object, '__self__', None)
        if oself is None: # pragma: no cover
            oself = getattr(object, 'im_self', None)
        return text_('method %s of class %s.%s' %
                     (object.__name__, modulename,
                      oself.__class__.__name__))

    if inspect.isclass(object):
        dottedname = '%s.%s' % (modulename, object.__name__)
        return text_('class %s' % dottedname)
    if inspect.isfunction(object):
        dottedname = '%s.%s' % (modulename, object.__name__)
        return text_('function %s' % dottedname)
    return text_('object %s' % str(object))



def ascii_encode_dict(data):
    """Cambia los caracteres a formato ASCII
       @example work
       json.loads(json_data, object_hook=helpers.ascii_encode_dict)

       json_data = '{"foo": "bar", "bar": "baz"}'
       >>> json.loads(json_data)                                # old call gives unicode
       {u'foo': u'bar', u'bar': u'baz'}
       >>> json.loads(json_data, object_hook=ascii_encode_dict) # new call gives str
       {'foo': 'bar', 'bar': 'baz'}
    """
    ascii_encode = lambda x: x.encode('ascii','ignore') if isinstance(x, unicode) else x
    return dict(map(ascii_encode, pair) for pair in data.items())


def json_encode_decimal(obj):
    """Transforma un objeto Decimal en un string

       @example work
       json.dumps(json_request, default=helpers.json_encode_decimal)
    """
    if isinstance(obj, Decimal):
        return str(obj)
    raise TypeError(repr(obj) + " is not JSON serializable")

def list2dict(keys=None,values=None):
    """A partir de dos listas, identificadas como <keys> y <values>, se genera un diccionario"""
    newDict = dict()
    try:
        if type(keys).__name__=='list' or type(keys).__name__=='tuple':
            if type(values).__name__=='list' or type(values).__name__=='tuple':
                if len(keys)==len(values):
                    newDict = dict(zip(keys, values))
                    return newDict
    finally:
        return newDict

def multikey2dict(data_dict,multikey_string):
    """Actualiza los datos de un dict o list - que puede estar compuesto a su vez por LIST y DICT hasta una profundidad de 4 -
       y que existe previamente, a partir de un dict con claves en formato <string-multikey>

       El formato <string-multikey> es un string compuesto de varias claves separadas entre si por puntos
       @example work
                      multikey2dict({ 'address':[ { 'number' : 0 }]}  , { address.0.number : 41} )

                      --> { 'address':[ { 'number' : 41 }]}

       <string-multikey> es un formato de clave mas apropiado para html form o aplanar dict
       Los string hacen referencia a claves dict y los numeros al orden en list.
    """
    for n in multikey_string.keys():
        g = n.split('.')
        v = multikey_string[n]
        if type(data_dict).__name__ == 'dict':
            # es un dict con clave g[0]
            if len(g)>1:
                if type(data_dict[g[0]]).__name__ == 'list':
                    # el siguiente valor clave corresponde a un int = list order
                    if type(data_dict[g[0]][int(g[1])]).__name__ == 'list':
                        if len(g)>2:
                            data_dict[g[0]][int(g[1])][int(g[2])] = v
                        else:
                            data_dict[g[0]][int(g[1])] = v
                    elif type(a[g[0]][int(g[1])]).__name__ == 'dict':
                        if len(g)>2:
                            data_dict[g[0]][int(g[1])][g[2]] = v
                        else:
                            data_dict[g[0]][int(g[1])] = v
                else:
                    # el siguiente valor clave corresponde a un string = dict key
                    if type(data_dict[g[0]][g[1]]).__name__ == 'list':
                        if len(g)>2:
                            data_dict[g[0]][g[1]][int(g[2])] = v
                        else:
                            data_dict[g[0]][g[1]] = v
                    elif type(a[g[0]][g[1]]).__name__ == 'dict':
                        if len(g)>2:
                            data_dict[g[0]][g[1]][g[2]] = v
                        else:
                            data_dict[g[0]][g[1]] = v
            else:
                a[g[0]] = v
        else:

            # es una list con calve int(g[0])
            if len(g)>1:
                if type(a[int(g[0])]).__name__ == 'list':
                    # el siguiente valor clave corresponde a un int = list order
                    if type(a[int(g[0])][int(g[1])]).__name__ == 'list':
                        if len(g)>2:
                            a[int(g[0])][int(g[1])][int(g[2])] = v
                        else:
                            a[int(g[0])][int(g[1])]
                    elif type(a[int(g[0])][int(g[1])]).__name__ == 'dict':
                        if len(g)>2:
                            a[int(g[0])][int(g[1])][g[2]] = v
                        else:
                            a[int(g[0])][int(g[1])] = v
                else:
                    # el siguiente valor clave corresponde a un string = dict key
                    if type(a[int(g[0])][g[1]]).__name__ == 'list':
                        if len(g)>2:
                            a[int(g[0])][g[1]][int(g[2])] = v
                        else:
                            a[int(g[0])][g[1]] = v
                    elif type(a[int(g[0])][g[1]]).__name__ == 'dict':
                        if len(g)>2:
                            a[int(g[0])][g[1]][g[2]] = v
                        else:
                            a[int(g[0])][g[1]] = v

            else:
                a[int(g[0])] = v
    return a

def dict2multikey(data_dict):
    """Transforma un dict o list - que puede estar compuesto por LIST y DICT hasta una profundidad de 4 -
       a un dict con claves en formato <string-multikey>

       El formato <string-multikey> es un string compuesto de varias claves separadas entre si por puntos

       @example work
                      dict2multikey({ 'address':[ { 'number' : 41 }]} )

                      --> { address.0.number : 41 }

       que es un formato mas apropiado para realizar html form
       Los string hacen referencia a claves dict y los numeros al orden en list.
    """
    ll = dict()
    if type(data_dict).__name__ == 'dict':
        # es un dict
        for n in data_dict.keys():
            if type(data_dict[n]).__name__ == 'dict':
                for m in data_dict[n].keys():
                    if type(data_dict[n][m]).__name__ == 'dict':
                        for l in data_dict[n][m].keys():
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([n,m,l])] = data_dict[n][m][l]

                    elif type(data_dict[n][m]).__name__ == 'list':
                        for l in range(len(data_dict[n][m])):
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([n,m,str(l)])] = data_dict[n][m][l]
                    else:
                        ll['.'.join([n,m])] = data_dict[n][m]

            elif type(data_dict[n]).__name__ == 'list':
                for m in range(len(data_dict[n])):
                    if type(data_dict[n][m]).__name__ == 'dict':
                        for l in data_dict[n][m].keys():
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([n,str(m),l])] = data_dict[n][m][l]
                    elif type(data_dict[n][m]).__name__ == 'list':
                        for l in range(len(data_dict[n][m])):
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([n,str(m),str(l)])] = data_dict[n][m][l]
                    else:
                        ll['.'.join([str(n),m])] = data_dict[n][m]
            else:
                ll[n] = data_dict[n]
    else:
        # es un list
        for n in range(len(data_dict)):
            if type(data_dict[n]).__name__ == 'dict':
                for m in data_dict[n].keys():
                    if type(data_dict[n][m]).__name__ == 'dict':
                        for l in data_dict[n][m].keys():
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([str(n),m,l])] = data_dict[n][m][l]

                    elif type(data_dict[n][m]).__name__ == 'list':
                        for l in range(len(data_dict[n][m])):
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([str(n),m,str(l)])] = data_dict[n][m][l]
                    else:
                        ll['.'.join([str(n),m])] = data_dict[n][m]

            elif type(data_dict[n]).__name__ == 'list':
                for m in range(len(data_dict[n])):
                    if type(data_dict[n][m]).__name__ == 'dict':
                        for l in data_dict[n][m].keys():
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([str(n),str(m),l])] = data_dict[n][m][l]

                    elif type(a[n][m]).__name__ == 'list':
                        for l in range(len(data_dict[n][m])):
                            if type(data_dict[n][m][l]).__name__ not in ['dict','list']:
                                ll['.'.join([str(n),str(m),str(l)])] = data_dict[n][m][l]
                    else:
                        ll['.'.join([str(n),m])] = data_dict[n][m]
            else:
                ll[str(n)] = data_dict[n]

    return ll

# Gestion de NUMEROS ###########################################################################################

def do_round(value, precision=0, method='common'):
    """Round the number to a given precision. The first
       parameter specifies the precision (default is ``0``), the
       second the rounding method:

       - ``'common'`` rounds either up or down
       - ``'ceil'`` always rounds up
       - ``'floor'`` always rounds down

       If you don't specify a method ``'common'`` is used.

          {{ 42.55|round }}
          -> 43.0
          {{ 42.55|round(1, 'floor') }}
          -> 42.5

       Note that even if rounded to 0 precision, a float is returned. If
       you need a real integer, pipe it through `int`:


       {{ 42.55|round|int }}
       -> 43
    """
    if not method in ('common', 'ceil', 'floor'):
        raise FilterArgumentError('method must be common, ceil or floor')
    if method == 'common':
        return round(value, precision)
    func = getattr(math, method)
    return func(value * (10 ** precision)) / (10 ** precision)


#esto es un test
if __name__ == '__main__' :
    print datetime2timestamp()
    g = encodeDateTime2String('2016031741906') #PQ5H8RXK
    print g
    k = decodeString2DateTime(g)       #2016031736936
    print type(k).__name__
    print k 
    k = decodeString2DateTime(g,dtime=True)
    print type(k).__name__
    print k
    h = datetime2timestamp(k)
    print h
    print timestamp2datetime(h)


    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (u'''Utilidades diversas''')
        print (u'''datetime2Timestamp(fecha o None) genera un entero timestamp que representa la fecha y hora actual o como parametro %d\n''' % datetime2timestamp())
        t = datetime2timestamp()
        print (u'''timestamp2Datetime(entero) devuelve una fecha a partir de un entero timestamp %s\n''' % timestamp2datetime(t))
        print (u'''Leer datos de un fichero de configuración read_config("inicio.cfg","USER","Title")\n''')
        print (u'''Guardar datos en un fichero de configuración save_config("inicio.cfg","SEGURO","Password","NadaDeNada")\n''')
        print (u'''Devuelve lista de palabras de un texto ordenado por el numero de apariciones
                  permite una lista de palabras a excluir del analisis get_keywords(texto,lista_de_palabras_a_excluir)\n''')
        print (u'''Cuentas las veces que una palabra aparece en un texto do_wordcount( text, word)\n''')
        print (u'''A partir de 2 listas se genera un diccionario list2dict(keys,values) keys=('primero','segundo') values=(1,2) %s\n''' % list2dict(('primero','segundo'),(1,2)))
