#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Modulo para identificar clientes conectados"""

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmai.com>'
__copyright__ = "Copyright (C) 2012 PYMETRICK.ORG"
__date__ = '2012-09-21'
__version__ = '0.4'
__license__ = 'Licencia GNU/GPLv2'
__credits__ = ''
__text__ = 'Identificacion de conexiones'
__file__ = 'session.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version
# 2014-08-30 v0.02 PL: - Second version
# 2015-09-10 v0.03 PL: - Adaptado a python 3.x


import sys
import os
import string
import stat
import base64
import hmac
import ipaddress
import binascii
import hashlib
import logging

try:
  import simplejson as json
except:
  import json

try:
    from itertools import izip as zip
except ImportError: # will be 3.x series
    pass

from datetime import date, time, datetime, timedelta
from random import SystemRandom,choice
from time import localtime,mktime
from helpers import *

# Caracteres permitidos en session
ALLOWED_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
# Para ofuscar la encriptacion de password
SALT_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
# Caracteres desordenados para generar una password aleatoria
PASSWORD_CHARS = "BCIOQSVXYZclovxz23478ADEFGHJKLMNPRTUW15690abdefghijkmnpqrstuwy"
# Numero de veces que se encriptara la password
DEFAULT_PBKDF2_ITERATIONS = 1000
# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;

# En Python 3  long = int
if PY3:
    long = int

# Comprobar codigo en desarrollo
DEBUG = False

# LOG para todas las funciones
__name__ = "session"
sys.stdout = sys.stderr
if DEBUG:
    logPath = ''
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)
    hdlr = logging.FileHandler(logPath+__name__+'.log')
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.info("Session modo local INICIO")
else:
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)


_builtin_safe_str_cmp = getattr(hmac, 'compare_digest', None)
_sys_rng = SystemRandom()



class Session:
    """Gestion de sessiones de conexion
    """
    __name__ = "session.Session"
    sys.stdout = sys.stderr

    if DEBUG:
        logPath = ''
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)
        hdlr = logging.FileHandler(logPath+__name__+'.log')
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        hdlr.setFormatter(formatter)
        LOG.addHandler(hdlr)
        LOG.info("Session modo local INICIO")
    else:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)

    def __init__(self, session_dir = None, key = "sessionID", expires = None, domain = None ):
        """constructor
        @Parameters
           session_dir :  Lugar se almacenan las de sesiones locales
           key : Nombre de la clave que identifica la sesión
           expires : Valor de la duracion de sesion en horas
           domain : Dominio al que afecta
        """
        try:
            self.__ip = None
            self.__session = dict()  # session values
            self.__session_dir = session_dir if(session_dir) else '.';
            self.__session_id = key if key else "sessionID";

            self.ipv6_address = ipaddress._BaseV6(None)
            # ipv4 localhost con norma ipv6 como int
            self.int_ipv4_localhost = int(self.ipv6_address._ip_int_from_string(str(ipaddress.ip_address('::ffff:%s' % '127.0.0.1'))))
            # ipv6 localhost como int
            self.int_ipv6_localhost = int(self.ipv6_address._ip_int_from_string('::1'))

            # tiempo de session en horas
            self.__expires = expires if(expires) else 5;

            # comprobar si existe almacen de session en el servidor
            self.is_session_dir()

            # crear estructura de self.__session
            self.__session[self.__session_id] = None
            self.__session['expires'] = None
            self.__session['short_expires'] = None
            self.__session['domain'] = None
            self.__session['looged_in'] = None
            self.__session['csrf_token'] = None
            # fichero que guardara la session del servidor
            self.__file_host = ''
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en __init__ <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def is_session_dir(self):
        """Comprueba si existe el almacen de sesiones
           o lo crea, si no existe.
        """
        try:
            if not os.path.exists(os.path.join(self.__session_dir)):
                os.mkdir(self.__session_dir)
                if PY3:
                    # para python 3
                    os.chmod(self.__session_dir , Oo660)
                elif PY2:
                    # para python 2
                    os.chmod(self.__session_dir , 0660)
                return True
            else:
                return True
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en is_session_dir <%s> en linea %s !!!' % (str(e),tb.tb_lineno))
            return False


    def ip_addr(self,environ = None):
        """Seleccionar la ip

        localhost en ipv4 127.0.0.1 y en ipv6 ::1

        @Parameters
             environ : dict que contiene los valores de una solicitud http
             Las claves a tratar son
             HTTP_CLIENT_IP o
             HTTP_X_FORWARDED_FOR o
             REMOTE_ADDR ( VALOR # 127.0.0.1  Y VALOR # NULO)
             Obteniendo la IPv4 o IPv6
        @Process
             si IPv4 convertir a notación IPv6  ::ffff:xxxx.xxxx.xxxx.xxxx y posteriormente a int
             si IPv6 convertir a int
        """
        self.__ip = None

        try:
            if type(environ).__name__ == "dict":
                # Buscar IP
                if environ.get("HTTP_CLIENT_IP",None) and self.__ip is None:
                  if len(environ['HTTP_CLIENT_IP'])>0:
                      # prevenir multiples direcciones ip
                      str_address = environ['HTTP_CLIENT_IP'].split(', ')[-1]
                      # corregir errores en direccion ip
                      addr = ipaddress.ip_address(str_address)
                      if addr.version == 4:
                          # convertir ipv4 a norma ipv6
                          addr = ipaddress.ip_address('::ffff:%s' % str_address)
                      addr = self.ipv6_address._ip_int_from_string(str(addr))
                      # comprobar si distinto de ip localhost
                      if int(addr) != self.int_ipv4_localhost and int(addr) != self.int_ipv6_localhost:
                          # seleccionar como origen ip en formato integer
                          self.__ip = int(addr)

                elif environ.get("HTTP_X_FORWARDED_FOR",None) and self.__ip == None:
                    if len(environ['HTTP_X_FORWARDED_FOR'])>0:
                        # prevenir multiples direcciones ip
                        str_address = environ['HTTP_X_FORWARDED_FOR'].split(', ')[-1]
                        # corregir errores en direccion ip
                        addr = ipaddress.ip_address(str_address)
                        if addr.version == 4:
                            # convertir ipv4 a norma ipv6
                            addr = ipaddress.ip_address('::ffff:%s' % str_address)
                        addr = self.ipv6_address._ip_int_from_string(str(addr))
                        # comprobar si distinto de ip localhost
                        if int(addr) != self.int_ipv4_localhost and int(addr) != self.int_ipv6_localhost:
                            # seleccionar como origen ip en formato integer
                            self.__ip = int(addr)


                elif environ.get("REMOTE_ADDR",None) and self.__ip == None:
                    if len(environ['REMOTE_ADDR'])>0:
                        # prevenir multiples direcciones ip
                        str_address = environ['REMOTE_ADDR'].split(', ')[-1]
                        # corregir errores en direccion ip
                        addr = ipaddress.ip_address(str_address)
                        if addr.version == 4:
                            # convertir ipv4 a norma ipv6
                            addr = ipaddress.ip_address('::ffff:%s' % str_address)
                        addr = self.ipv6_address._ip_int_from_string(str(addr))
                        # comprobar si distinto de ip localhost
                        if int(addr) != self.int_ipv4_localhost and int(addr) != self.int_ipv6_localhost:
                            # seleccionar como origen ip en formato integer
                            self.__ip = int(addr)

            if self.__ip is None:
                self.__ip = self.int_ipv6_localhost

        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en ip_addr <%s> en linea %s !!!' % (str(e),tb.tb_lineno))


    def set_environ(self, environ=None):
        """Tratamiento del entorno http

        @Parameters
           environ : dict que contiene los valores de una solicitud http
             Las claves a tratar son
             HTTP_COOKIE
        """
        try:

            self.ip_addr(environ)

            # buscar valor cookie + IP
            dict_cookie = dict()
            if type(environ).__name__ == "dict":
                if self.is_session_dir():
                    if 'HTTP_COOKIE' in environ:
                          # identificar y separar en dict los valores de HTTP_COOKIE
                          list_cookie = environ['HTTP_COOKIE'].split(';')
                          # obtener clave y valor
                          if len(list_cookie)>0:
                              for n in list_cookie:
                                  # separar por valores por =
                                  l = n.encode('utf8').split('=')
                                  if len(l)>1:
                                      # obtener clave y valor
                                      dict_cookie[l[0].strip()] = l[1].strip()
                                  elif len(l)==1:
                                      # obtener solo clave
                                      dict_cookie[l[0].strip()] = None
    
                          # buscar key session en cookie
                          if dict_cookie.get(self.__session_id,None):
                              # comprobar identificador de session
                              if len(dict_cookie[self.__session_id].strip())>0:
                                  # comprobar fichero <session + ip> de session
                                  self.__file_host = '_'.join([dict_cookie[self.__session_id],str(self.__ip)])
                                  if os.path.exists(os.path.join(self.__session_dir, self.__file_host)):
                                      # existe session
                                      self.__session[self.__session_id] = dict_cookie[self.__session_id]
                                      if not self.load_session():
                                          self.LOG.error('ERROR Session %s no se ha recuperado' % self.__session[self.__session_id])
                                      # comprobar si tiempo expirado
                                      if self.is_expires():
                                          # crear nueva session
                                          self.new_session()
                                          # crear dominio
                                          self.__session['domain'] = '.'.join(environ['HTTP_HOST'].split('.')[-2:])
                                          if not self.save_session():
                                              self.LOG.error('ERROR Session %s no se ha guardado' % self.__session[self.__session_id])
                                  else:
                                      # crear nueva session
                                      self.__session[self.__session_id] = dict_cookie[self.__session_id]
                                      # crear dominio
                                      self.__session['domain'] = '.'.join(environ['HTTP_HOST'].split('.')[-2:])
                                      self.set_expires()
                                      if not self.save_session():
                                          self.LOG.error('ERROR Session %s no se ha guardado' % self.__session[self.__session_id])
                          else:
                              # No existe key session en cookie
                              self.new_session()
                              # crear dominio
                              self.__session['domain'] = '.'.join(environ['HTTP_HOST'].split('.')[-2:])
                              if not self.save_session():
                                  self.LOG.error('ERROR Session %s no se ha guardado' % self.__session[self.__session_id])

                    else:
                        # no existe HTTP_COOKIE
                        # crea nueva key session
                        self.new_session()
                        # crear dominio
                        self.__session['domain'] = '.'.join(environ['HTTP_HOST'].split('.')[-2:])
                        if not self.save_session():
                            self.LOG.error('ERROR Session %s no se ha guardado' % self.__session[self.__session_id])
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en set_environ <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def new_session(self):
        """Crear nueva key session """
        try:
            sid = None
            while sid is None:
                sid = ''.join([csrf_token(48),str(int(mktime(datetime.now().timetuple())))])
                sid = ''.join([i for i in sid if i in ALLOWED_CHARS])
                """Comprueba existencia de almacen de sessiones o en su defecto lo crea"""
                if self.is_session_dir():
                    # comprobar si existe valor de session + IP en servidor
                    self.__file_host = '_'.join([sid,str(self.__ip)])
                    if os.path.exists(os.path.join(self.__session_dir, self.__file_host)):
                        # ya existe y debe volver a calcular un nuevo valor de session
                        sid = None
                    else:
                        self.__session[self.__session_id] = sid
                        self.__session['csrf_token']= csrf_token()
                        self.set_expires()
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en new_session <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def save_session(self):
        """Graba los valores de session"""
        try:
            if self.__file_host:
                # grabar datos de session
                out_file = open(os.path.join(self.__session_dir, self.__file_host ),'w')
                out_file.write(json.dumps(self.__session))
                out_file.close()
                return True
            else:
                return False
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en save_session <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def load_session(self):
        """Lee los valores de session
           y los deja disponibles en formato diccionario
        """
        try:
            if self.__file_host:
                if os.path.exists(os.path.join(self.__session_dir, self.__file_host)):
                    # lee datos de session
                    in_file = open(os.path.join(self.__session_dir, self.__file_host)).read()
                    self.__session = json.loads(in_file, object_hook=ascii_encode_dict)
                    return True
                else:
                    return False
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en load_session <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def set_expires(self):
        """tiempo de vida de session en en horas (hours)"""
        try:
            expiration = datetime.utcnow() + timedelta(hours=self.__expires)
            self.__session['expires'] = expiration.strftime("%a, %d-%b-%Y %H:%M:%S UTC")
            self.__session['short_expires'] = expiration.strftime("%Y%m%d%H%M%S")
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en set_expires <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def is_expires(self):
        """comprueba si tiempo ha expirado"""
        try:
            if self.__session.get('short_expires',None):
                if self.__session['short_expires']<= datetime.utcnow().strftime("%Y%m%d%H%M%S"):
                    # elimina fichero de session en servidor
                    if not self.remove_session():
                        self.LOG.error('ERROR Session %s no se ha eliminado' % self.__session[self.__session_id])
                    # tiempo de session finalizado
                    return True
                else:
                    return False
            else:
                return False
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en is_expires <%s> en linea %s !!!' % (str(e),tb.tb_lineno))


    def is_ip(self,ip=None):
        """Verificacion de ip de conexion con norma ipv6"""
        try:
            if ip:
                addr = ipaddress.ip_address(ip)
                if addr.version == 4:
                    # convertir ipv4 a norma ipv6
                    addr = ipaddress.ip_address('::ffff:%s' % addr)
                addr = self.ipv6_address._ip_int_from_string(str(addr))
                # comprobar si distinto de ip localhost
                if int(addr) != self.__ip:
                    return False
                else:
                    return True
            else:
                return False
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en is_ip <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def get_ip(self):
        """Devolver ip de conexion"""
        try:
            if self.__ip:
                return self.__ip
            else:
                return None
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en is_ip <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def set_session(self,key = None, key_value = None):
        '''Incorporar clave y valor a una session
        @Parameters
            key :  debe ser una clave permitida
            value : valor de la clave
        '''
        try:
            if key in self.__session:
                if self.load_session():
                    self.__session[key] = key_value
                    if self.save_session():
                        return True
                    else:
                        return False
                else:
                    return False
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en set_session <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def get_session(self):
        """devuelve identificacion de session
           en formato diccionario
        """
        try:
            if self.load_session():
                return self.__session
            else:
                return None
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en get_session <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def set_cookie(self):
        '''prepara informacion para introducir en cookie'''
        self.load_session()
        return ("sessionID=%s; Domain=%s; path=/; expires=%s" % (self.__session["sessionID"],'.'+self.__session["domain"],self.__session["expires"])).encode('utf8')

    def remove_session(self):
        '''Elimina fichero de session

        @Parameters
              None
        @Return
              True si tiene exito
        '''
        try:
            if os.path.exists(os.path.join(self.__session_dir, self.__file_host)):
                os.remove(os.path.join(self.__session_dir, self.__file_host))
                return True
            else:
                return False
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en remove_session <%s> en linea %s !!!' % (str(e),tb.tb_lineno))
            return False

    def __del__(self):
        '''Eliminar objeto y objetos dependientes'''
        if DEBUG:
            self.LOG.debug("del ha eliminado el objeto")
        else:
            pass


def safe_str_cmp(a, b):
    """This function compares strings in somewhat constant time.  This
    requires that the length of at least one string is known in advance.
    Returns `True` if the two strings are equal or `False` if they are not.
    """
    try:
        if _builtin_safe_str_cmp is not None:
            return _builtin_safe_str_cmp(a, b)
        if len(a) != len(b):
            return False
        rv = 0
        if isinstance(a, bytes) and isinstance(b, bytes) and PY3:
            for x, y in zip(a, b):
                rv |= x ^ y
        else:
            for x, y in zip(a, b):
                rv |= ord(x) ^ ord(y)
        return rv == 0
    except Exception as e:
	LOG.error('Error session.safe_str_cmp %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))


def gen_salt(length = 8):
    """Genera una cadena aleatoria de SALT_CHARS
       @Parameters
           length : longitud de cadena de caracteres a obtener
       @Return
            una cadena de caracteres aleatoria

    """
    try:
        if length <= 0:
            raise ValueError('requested salt of length <= 0')
        return (''.join(_sys_rng.choice(SALT_CHARS) for _ in range(length))).strip()
    except Exception as e:
	LOG.error('Error session.gen_salt %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def _find_hashlib_algorithms():
    try:
        algos = getattr(hashlib, 'algorithms', None)
        if algos is None:
            algos = ('md5', 'sha1', 'sha224', 'sha256', 'sha384', 'sha512')
        # mostrar metodos disponibles
        rv = {}
        for algo in algos:
            func = getattr(hashlib, algo, None)
            if func is not None:
                rv[algo] = func
        return rv
    except Exception as e:
	LOG.error('Error session._find_hashlib_algorithms %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

_hash_funcs = _find_hashlib_algorithms()

def pbkdf2_hex(data, salt, iterations=DEFAULT_PBKDF2_ITERATIONS,
               keylen=None, hashfunc=None):
    """como la funcion 'pbkdf2_bin' pero devuelve una cadena hexadecimal.

    @Parameters
        data: the data to derive.
        salt: the salt for the derivation.
        iterations: the number of iterations.
        keylen: the length of the resulting key.  If not provided
                   the digest size will be used.
        hashfunc: the hash function to use.  This can either be the
                     string name of a known hash function or a function
                     from the hashlib module.  Defaults to sha1.
    """
    try:
        rv = pbkdf2_bin(data, salt, iterations, keylen, hashfunc)
        return to_native(codecs.encode(rv, 'hex_codec'))
    except Exception as e:
	LOG.error('Error session.pbkdf2_hex %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))


def pbkdf2_bin(data, salt, iterations=DEFAULT_PBKDF2_ITERATIONS,
               keylen=None, hashfunc=None):
    """Returns a binary digest for the PBKDF2 hash algorithm of `data`
    with the given `salt`. It iterates `iterations` time and produces a
    key of `keylen` bytes. By default SHA-1 is used as hash function,
    a different hashlib `hashfunc` can be provided.

    @Parameters
        data: the data to derive.
        salt: the salt for the derivation.
        iterations: the number of iterations.
        keylen: the length of the resulting key.  If not provided
                   the digest size will be used.
        hashfunc: the hash function to use.  This can either be the
                     string name of a known hash function or a function
                     from the hashlib module.  Defaults to sha1.
    """
    try:
        if isinstance(hashfunc, string_types):
            hashfunc = _hash_funcs[hashfunc]
        elif not hashfunc:
            hashfunc = hashlib.sha1
        salt = to_bytes(salt)
        mac = hmac.HMAC(to_bytes(data), None, hashfunc)
        if not keylen:
            keylen = mac.digest_size
        def _pseudorandom(x, mac=mac):
            h = mac.copy()
            h.update(x)
            return bytearray(h.digest())
        buf = bytearray()
        for block in range_type(1, -(-keylen // mac.digest_size) + 1):
            rv = u = _pseudorandom(salt + _pack_int(block))
            for i in range_type(iterations - 1):
                u = _pseudorandom(bytes(u))
                rv = bytearray(starmap(xor, izip(rv, u)))
            buf.extend(rv)
        return bytes(buf[:keylen])
    except Exception as e:
	LOG.error('Error session.pbkdf2_bin %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def _hash_internal(method, salt , password):
    """Contraseña Interna ayudante hash. Soporta texto claro sin ofuscar,
       contraseñas sin ofuscar y ofuscada. En el caso de contraseñas ofuscadas se utiliza
       HMAC.
    @Parameters
        method : metodo para encriptar la password, permite devolver el texto sin encriptar con 'plain'
        salt   : cadema de caracteres que permitira ofuscar los caracteres que componen la password
        password : texto que deseamos tratar.
    @Return
        tuple( texto de password encriptado y/o ofuscado, metodo de encriptacion)
    """
    try:
        if method == 'plain':
            return password, method

        password = password.encode('utf-8')

        if method.startswith('pbkdf2:'):
            args = method[7:].split(':')
            if len(args) not in (1, 2):
                raise ValueError('Invalid number of arguments for PBKDF2')
            method = args.pop(0)
            iterations = args and int(args[0] or 0) or DEFAULT_PBKDF2_ITERATIONS
            is_pbkdf2 = True
            actual_method = 'pbkdf2:%s:%d' % (method, iterations)
        else:
            is_pbkdf2 = False
            actual_method = method

        hash_func = _hash_funcs.get(method)
        if hash_func is None:
            raise TypeError('invalid method %r' % method)

        if is_pbkdf2:
            if not salt:
                raise ValueError('Salt is required for PBKDF2')
            rv = pbkdf2_hex(password, salt, iterations,
                            hashfunc=hash_func)
        elif salt:
            salt = salt.encode('utf-8')
            rv = hmac.HMAC(salt, password, hash_func).hexdigest()
        else:
            h = hash_func()
            h.update(password)
            rv = h.hexdigest()
        return rv, actual_method
    except Exception as e:
	LOG.error('Error session._hash_internal %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def generate_password_hash(password, method='sha1', salt_length=8):
    """Encripta una contraseña con el método dado y derivada de ofuscacion con la longitud
    dada. El formato de la cadena devuelta incluye el método que se utilizó de forma que
    `check_password_hash` pueda comprobar el hash.

    El formato de la cadena hash se ve así::

        method$salt$hash

    Este método no puede generar contraseñas sin ofuscacion, pero es posible
    generar las contraseñas en texto plano. Si utiliza derivada de ofuscacion,
    HMAC se utiliza internamente para ofuscar la contraseña.

    @parameters
        password: cadena de caracteres a encriptar
        method: metodo de encriptacion empleado (soportado por hashlib)
        salt_length: numero de caracteres usados de ofuscar la contraseña
    """
    try:
        salt = method != 'plain' and gen_salt(salt_length) or ''
        h, actual_method = _hash_internal(method, salt, password)
        return '%s$%s$%s' % (actual_method, salt, h)
    except Exception as e:
	LOG.error('Error session.generate_password_hash %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))


def check_password_hash(pwhash, password):
    """comprobar una contraseña con un valor de contraseña dada salada y hash.
    Con el fin de apoyar las contraseñas sin sal heredados Este método admite
    contraseñas de texto sin formato MD5 y SHA1 hashes (tanto con y sin sal).

    Returns `True` if the password matched, `False` otherwise.
    @Parameters
        pwhash: una password encriptada como la que se genera desde generate_password_hash()
        password: texto sin encriptar, correspondiente a la password, para comparar.
    @Return
        'True' si la password coincide o 'False' en cualquier otro caso.
    """
    try:
        if pwhash.count('$') < 2:
            return False
        method, salt, hashval = pwhash.split('$', 2)
        return safe_str_cmp(_hash_internal(method, salt, password)[0], hashval)
    except Exception as e:
	LOG.error('Error session.check_password_hash %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def csrf_token(csrf_length=32):
    """genera un string con valores aleatorios con una longitud determinada
       orientado al Cross Site Request Forgery protection
       puede ser usado con las comunicaciones ajax
       en la sesion del servidor deberá indicarse como csrftoken y se indicara
       el valor aleatorio en cada pagina generada, recibiendo el mismo valor
       cuando se realice una solicitud ajax o envío de un formulario GET, POST,
       PUT, DELETE.
       @Parameters
           csrf_length : longitud de la cadena de caracteres a obtener
       @Return
           cadena de caracteres aleatoria con la longitud solicitada
    """
    try:
        return (gen_salt(csrf_length)).encode('utf-8')
    except Exception as e:
	LOG.error('Error session.csrf_token %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def new_password(length = 15, chars = None):
    """Genera password aleatoria dependiendo de la cadena de caracteres recibida o cadena por defecto
       @Parameters
                length = longitud de password
                chars  = cadena de caracteres para generacion aleatoria de password, aunque por defecto
                         puede utilizar una cadena propia
       @Return
                String = password sin encriptacion de la longitud solicitada u 8 caracteres por defecto
    """
    try:
        local_chars = PASSWORD_CHARS if(chars) is None else chars;
        if len(local_chars)<30:
            # si la cadena recibida contiene menos de 30 caracteres se amplia de letras y numeros
            local_chars = string.letters + string.digits
        return ''.join(choice(local_chars) for _ in range(length))
    except Exception as e:
	LOG.error('Error session.new_password %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

def file2checksums(__path__, blocksize=65536):
    """obtener el hash de un fichero para identificarlo de forma unica
    """
    try:
	hash = hashlib.md5()
	with open(__path__, "rb") as f:
	    for block in iter(lambda: f.read(blocksize), b""):
		hash.update(block)
	return hash.hexdigest()
    except Exception as e:
	LOG.error('Error session.file2checksums %s in line %s' % (e,format(sys.exc_info()[-1].tb_lineno)))

if __name__ == "__main__":
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help\n")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print(u"Identificacion de clientes conectados\n")


