#!/usr/bin/python
# -*- coding: utf-8 *-*

"""Modulo para gestionar datos en MySQL"""

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmail.com>'
__copyright__ = "Copyright (C) 2015 PYMETRICK.ORG"
__date__ = '2015-06-26'
__version__ = '0.4'
__license__ = 'Licencia GNU/GPLv2'
__credits__ = ''
__text__ = 'Gestion de datos en BBDD'
__file__ = 'sqldb.py'

#--- CHANGES ------------------------------------------------------------------
# 2015-06-26 v0.01 PL: - First version
# 2015-09-10 v0.03 PL: - Adaptado a python 3.x


import sys
import os
import re
import time
import logging
#import importlib
import mysql.connector
from mysql.connector.errorcode import *

RETRY_NUMBER = 50   # Cuantas veces intenta reconectar
RETRY_SLEEP  = 60  # Tiempo de espera

# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;

if PY2:
    # python3 chr & python2 unichr
    chr = unichr
    
if PY3:
    long = int

# Comprobar codigo en desarrollo
DEBUG = False

class SQLdb(object):
    __instance__ = None

    #Tratamiento de errores en LOG
    __name__ = "SQLdb"
    sys.stdout = sys.stderr

    if DEBUG:
        logPath = ''
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)
        hdlr = logging.FileHandler(logPath+__name__+'.log')
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        hdlr.setFormatter(formatter)
        LOG.addHandler(hdlr)
        LOG.info("Session modo local INICIO")
    else:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)

    start_time = time.time()
    query_time = 0

    _module = None
    _errorcode = None
    cnx = None
    cursor = None


    value_type = {'CHAR':'text',
                 'VARCHAR':'text',
                 'TINYBLOB':'text',
                 'TINYTEXT':'text',
                 'BLOB':'text',
                 'TEXT':'text',
                 'MEDIUMBLOB':'text',
                 'MEDIUMTEXT':'text',
                 'LONGBLOB':'text',
                 'LONGTEXT':'text',
                 'TINYINT':'int',
                 'SMALLINT':'int',
                 'MEDIUMINT':'int',
                 'INT':'int',
                 'INTEGER':'int',
                 'BIGINT':'long',
                 'FLOAT':'float',
                 'DOUBLE':'float',
                 'REAL':'float',
                 'DECIMAL':'decimal',
                 'NUMERIC':'decimal',
                 'DATE':'date',
                 'DATETIME':'datetime',
                 'TIMESTAMP':'datetime',
                 'TIME':'float',
                 'YEAR':'int'}

    field_type = {0: 'DECIMAL',
                  1: 'TINY',
                  2: 'SHORT',
                  3: 'LONG',
                  4: 'FLOAT',
                  5: 'DOUBLE',
                  6: 'NULL',
                  7: 'TIMESTAMP',
                  8: 'LONGLONG',
                  9: 'INT24',
                 10: 'DATE',
                 11: 'TIME',
                 12: 'DATETIME',
                 13: 'YEAR',
                 14: 'NEWDATE',
                 15: 'VARCHAR',
                 16: 'BIT',
                246: 'NEWDECIMAL',
                247: 'INTERVAL',
                248: 'SET',
                249: 'TINY_BLOB',
                250: 'MEDIUM_BLOB',
                251: 'LONG_BLOB',
                252: 'BLOB',
                253: 'VAR_STRING',
                254: 'STRING',
                255: 'GEOMETRY' }


    def __new__(cls, *args, **kargs):
        if cls.__instance__ is None:
            cls.__instance__ = object.__new__(cls, *args, **kargs)
        return cls.__instance__


    def __init__(self,*args, **kwargs):
        """Abrir acceso a bbdd"""
        self.cnx = None;
        self.cursor = None;
        self._host = kwargs['host'] if kwargs.get('host',None) else '127.0.0.1';
        self._port = kwargs['port'] if kwargs.get('port',None) else '3306';
        self._username = kwargs['username']
        self._password = kwargs['password']
        self._database = kwargs['database']
        self._driver = kwargs['driver'] if kwargs.get('driver',None) else 'mysql';
        self._max_idle_time = kwargs['max_idle_time'] if kwargs.get('max_idle_time',None) else 2*3600;
        self._time_zone = kwargs['time_zone'] if kwargs.get('time_zone',None) else "+2:00";
        self._sql_mode = kwargs['sql_mode'] if kwargs.get('sql_mode',None) else "TRADITIONAL";
        self._charset = kwargs['charset'] if kwargs.get('charset',None) else "utf8";
        self._collation = kwargs['collation'] if kwargs.get('collation',None) else "utf8_spanish_ci";
        self._conection_timeout = kwargs['connection_timeout'] if kwargs.get('connection_timeout',None) else 100;
        self._compress = kwargs['compress'] if kwargs.get('compress',None) else False;
        self._autocommit = kwargs['autocommit'] if kwargs.get('autocommit',None) else False;
        self._use_pure = kwargs['use_pure'] if kwargs.get('use_pure',None) else True;
        self._use_unicode = kwargs['use_unicode'] if kwargs.get('use_unicode',None) else True;

        #encloser characters for system identifiers(key_delim) and strings(str_delim)
        self._key_delim = '"';
        self._str_delim = "'";

        if self._driver == 'mysql':
            #self._module = importlib.import_module('mysql.connector')
            #self._errorcode = importlib.import_module("mysql.connector","errorcode")
            self._key_delim = '`';
        elif self._driver == 'pgsql':
            pass
            #self._module = importlib.import_module('psycopg2')
        else:
            self.LOG.error('Driver Error Unknown database driver !!!')
            raise Exception("Unknown database driver")



        # self.rows permite obtener las filas afectadas
        self.rows = 0
        self.lastId = 0

        self._last_use_time = time.time()
        try:
            if self.reconnect():
                if getattr(self,"cursor",None) is not None:
                    self.cursor.close()
                    self.cursor = None
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error %s.__init__ <%s> en linea %s !!!' % (self.__name__,str(e),tb.tb_lineno))

    def __del__(self):
        self.close()

    def close(self):
        """Cierra la conexion a la base de datos"""
        try:
            if getattr(self,"cnx",None) is not None:
                if getattr(self,"cursor",None) is not None:
                    self.cursor.close()
                    self.cursor = None
                self.cnx.close()
                self.cnx = None
        except mysql.connector.Error as err:
                self.LOG.error('ERROR %s.close %s' % ( self.__name__,err))

    def reconnect(self):
        """Cierra la conexion con la base de datos y vuelve a abrirla"""
        try:
            self.close()
            config = {'host': self._host, 'port': self._port, 'user': self._username};
            if self._driver == 'mysql':
                config['password'] = self._password
                config['database'] = self._database
                config['time_zone']= self._time_zone
                config['charset'] = self._charset
                config['collation'] = self._collation
                config['sql_mode']= self._sql_mode
                config['compress']= self._compress
                config['autocommit']= self._autocommit
                config['raise_on_warnings']= True
                #config['use_pure'] = self._use_pure
                config['use_unicode'] = self._use_unicode
                self.cnx = mysql.connector.connect(**config)
                self.cursor = self.cnx.cursor()                
                set_wait_timeout
            elif self._driver == 'pgsql':
                # SI EL
                config['database'] = self._dbname
                config['password'] = self._password
                # pendiente de conexion a pqsgl
            return True
        except mysql.connector.Error as err:
            if err.errno == ER_ACCESS_DENIED_ERROR:
                self.LOG.error('ERROR %s.reconnect Something is wrong with your user name or password !!!' % self.__name__)
            elif err.errno == ER_BAD_DB_ERROR:
                self.LOG.error('ERROR %s.reconnect Database does not exist !!!' % self.__name__)
            else:
                self.LOG.error('ERROR %s.reconnect %s' % ( self.__name__,err))
            return False

    def execute(self, sql=None , parameters=None):
        """Ejecuta consultas teniendo en cuenta transacciones
        sql parametro con el codigo
        parameters son las variables que se pueden incorporar al codigo sql
        """
        self.ensure_connected()
        try:
            if sql:
                if parameters is not None:
                    if DEBUG:
                        self.LOG.debug('query %s' % (sql % parameters))
                    if type(parameters[0]).__name__ == 'tuple':
                        self.rows = self.cursor.executemany(sql, parameters)
                    else:
                        self.rows = self.cursor.execute(sql, parameters)
                else:
                    if DEBUG:
                        self.LOG.debug('query %s' % (sql))
                    self.rows = self.cursor.execute(sql)
                if sql[0:6].upper()=='INSERT':
                    # Informa de nº de última fila autoincremental
                    self.lastId = self.cursor.lastrowid
                    return self.lastId

                elif sql[0:6].upper()=='UPDATE' or sql[0:6].upper()=='DELETE':
                    # Informa de nº filas afectadas
                    self.rows = self.cursor.rowcount
                    return self.rows

                elif sql[0:6].upper()=='SELECT':
                    # Devuelve informacion de todas las filas seleccionadas
                    return self.cursor.fetchall()

        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.execute %s' % ( self.__name__,err))
        finally:
            if hasattr(self.cnx,"commit"):
                self.cnx.commit()
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def set_auto_increment(self,table=None,id=1):
        """Fija el autoincremento en la tabla correspondiente al parametro"""
        self.ensure_connected()
        try:
            if table is not None:
                self.cursor.execute('ALTER TABLE %s AUTO_INCREMENT = %d' % (table,id))
            return True
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.set_auto_increment %s' % ( self.__name__,err))
            return False
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None
                
    def set_wait_timeout(self,seconds=60):
        """Fija wait_timeout y interactive_timeour a 60 por defecto"""
        self.ensure_connected()
        try:
            if seconds is not None:
                self.cursor.execute('set @@wait_timeout=%s' % seconds)
                self.cursor.execute('set @@interactive_timeout=%s' % seconds)
                if self.cursor.execute('select @@wait_timeout')[0]==seconds and self.cursor.execute('select @@interactive_timeout')[0]==seconds:
                    return True
                else:
                    return False
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error %s.__init__ <%s> en linea %s !!!' % (self.__name__,str(e),tb.tb_lineno))
            return False
        
    
    def set_optimize(self,table=None):
        """Optimizacion de tablas INNODB"""
        self.ensure_connected()
        try:
            if table is not None:
                self.cursor.execute('OPTIMIZE TABLE %s' % table)
            return True
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.set_optimize %s' % ( self.__name__,err))
            return False
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def show_version(self):
        """Version de MySQL"""
        self.ensure_connected()
        try:
            self.cursor.execute('select version();')
            return self.cursor.fetchall()
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.show_version %s' % ( self.__name__,err))
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def create_database(self,database):
        """Crear una base de datos"""
        self.ensure_connected()
        ddl = "CREATE DATABASE IF NOT EXISTS %s" %(name)
        try:
            self.cursor.execute(ddl)
        except mysql.connector.Error as err:
            if err.errno == ER_CANT_CREATE_DB:
                self.LOG.error('ERROR %s.create_database can´t create db %s' % ( self.__name__,err))
            else:
                self.LOG.error('ERROR %s.create_database %s' % ( self.__name__,err))
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def create_table(self,table):
        """Crear una tabla"""
        self.ensure_connected()
        try:
            pass
        except mysql.connector.Error as err:
            if err.errno == ER_CANT_CREATE_TABLE:
                self.LOG.error('ERROR %s.create_table can´t create table %s' % ( self.__name__,err))
            else:
                self.LOG.error('ERROR %s.create_table %s' % ( self.__name__,err))


    def show_tables(self):
        """Ver lista de tablas"""
        self.ensure_connected()
        try:
            self.cursor.execute('show tables;')
            return self.cursor.fetchall()
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.show_tables %s' % ( self.__name__,err))
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def show_columns(self, table=''):
        """Ver lista de columnas"""
        self.ensure_connected()
        try:
            self.cursor.execute('show columns from %s;' % table)
            return self.cursor.fetchall()
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.show_columns %s' % ( self.__name__,err))
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def show_index(self, table=''):
        """Ver lista de indices"""
        self.ensure_connected()
        try:
            self.ensure_connected()
            self.cursor.execute('show index from %s;' % table)
            return self.cursor.fetchall()
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.show_index %s' % ( self.__name__,err))
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None


    def show_vars(self):
        """Ver lista de variables"""
        self.ensure_connected()
        try:
            self.ensure_connected()
            self.cursor.execute('show variables;')
            return self.cursor.fetchall()
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.show_vars %s' % ( self.__name__,err))
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def show_process(self):
        """Ver lista de procesos del usuario"""
        self.ensure_connected()
        try:
            self.cursor.execute('show processlist where user like %s;' % self._db_args['user'])
            return self.cursor.fetchall()
        except mysql.connector.Error as err:
            self.LOG.error('ERROR %s.show_process %s' % ( self.__name__,err))
        finally:
            if getattr(self,"cursor",None) is not None:
                self.cursor.close()
                self.cursor = None

    def get_value_type(self):
        """Devolver tipo de valores admitidos por MySQL"""
        return self.value_Type

    def ensure_connected(self):
        """ Mysql por defecto cierra las conexiones de clientes que están inactivos por
        8 horas, pero la biblioteca de cliente no reporta este hecho hasta
        intentar realizar una nueva consulta y en ese momento falla.
        Preventivamente se cierra y vuelve a abrir la conexion si no se ha utilizado
        durante mucho tiempo"""
        retry = 0
        while retry < RETRY_NUMBER:
            try:
                if (getattr(self,"cursor",None) is not None or (time.time() - self._last_use_time > self._max_idle_time)):
                    self.reconnect()
                else:
                    self.cursor = self.cnx.cursor()
                self._last_use_time = time.time()
                # si la reconexion es correcta
                retry = RETRY_NUMBER
            except mysql.connector.Error as err:
                if err.errno == CR_CONNECTION_ERROR:
                    self.LOG.error('ERROR %s.ensure_connected %s' % ( self.__name__,err))
                elif err.errno == CR_LOCALHOST_CONNECTION:
                    self.LOG.error('ERROR %s.ensure_connected %s' % ( self.__name__,err))
                else:
                    self.LOG.error('ERROR %s.ensure_connected %s' % ( self.__name__,err))
                time.sleep(RETRY_SLEEP)

    def __del__(self):
        '''Eliminar objeto y objetos dependientes'''
        if DEBUG:
            self.LOG.debug("del ha eliminado el objeto")
        else:
            pass


if __name__ == "__main__":
    pass

