#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Modulo para gestionar WSGI"""

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmail.com>'
__copyright__ = "Copyright (C) 2014 PYMETRICK.ORG"
__date__ = '2014-05-21'
__version__ = '0.4'
__license__ = 'Licencia GNU/GPLv2'
__credits__ = ''
__text__ = 'Gestion de funciones para WSGI'
__file__ = 'wsgi.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version

import sys
import os
import base64
import logging
try:
  import simplejson as json
except:
  import json
  
from helpers import *

# Version PY
PY2 = True if sys.version[:1] == '2' else False;
PY3 = True if sys.version[:1] == '3' else False;

# En Python 3  long = int
if PY3:
    long = int

# Comprobar codigo en desarrollo
DEBUG = False

CONTENT_TYPE = {"application/javascript": "request",
                "application/json" : "request_json",
                "application/mp4": "request",
                "application/msword": "request",
                "application/pdf": "request",
                "application/rtf": "request",
                "application/zip": "request",
                "audio/midi": "request",
                "audio/mp4": "request",
                "audio/mpeg": "request",
                "audio/ogg": "request",
                "image/bmp": "request",
                "image/gif": "request",
                "image/jpeg": "request",
                "image/png": "request",
                "image/tiff": "request",
                "multipart/form-data": "request_multipart",
                "text/calendar": "request",
                "text/css": "request",
                "text/csv": "request",
                "text/html": "request",
                "text/plain" : "request",
                "video/jpeg": "request",
                "video/mp4": "request",
                "video/mpeg": "request",
                "video/ogg": "request",
                "video/quicktime": "request"}

STATUS_CODE = {
    "100": {
        "phrase": "Continue",
        "description": "\"indicates that the initial part of a request has been received and has not yet been rejected by the server.\"",
    },
    "101" : {
        "phrase": "Switching Protocols",
        "description": "\"indicates that the server understands and is willing to comply with the client's request, via the Upgrade header field, for a change in the application protocol being used on this connection.\"",
    },
    "102" : {
        "phrase": "Processing",
        "description": "\"is an interim response used to inform the client that the server has accepted the complete request, but has not yet completed it.\"",
    },
    "200" : {
        "phrase": "OK",
        "description": "\"indicates that the request has succeeded.\"",
    },
    "201" : {
        "phrase": "Created",
        "description": "\"indicates that the request has been fulfilled and has resulted in one or more new resources being created.\"",
    },
    "202" : {
        "phrase": "Accepted",
        "description": "\"indicates that the request has been accepted for processing, but the processing has not been completed.\"",
    },
    "203" : {
        "phrase": "Non-Authoritative Information",
        "description": "\"indicates that the request was successful but the enclosed payload has been modified from that of the origin server's 200 (OK) response by a transforming proxy.\"",
    },
    "204" : {
        "phrase": "No Content",
        "description": "\"indicates that the server has successfully fulfilled the request and that there is no additional content to send in the response payload body.\"",
    },
    "205" : {
        "phrase": "Reset Content",
        "description": "\"indicates that the server has fulfilled the request and desires that the user agent reset the \"document view\", which caused the request to be sent, to its original state as received from the origin server.\"",
    },
    "206" : {
        "phrase": "Partial Content",
        "description": "\"indicates that the server is successfully fulfilling a range request for the target resource by transferring one or more parts of the selected representation that correspond to the satisfiable ranges found in the requests's Range header field.\"",
    },
    "207" : {
        "phrase": "Multi-Status",
        "description": "\"provides status for multiple independent operations.\"",
    },
    "226" : {
        "phrase": "IM Used",
        "description": "\"The server has fulfilled a GET request for the resource, and the response is a representation of the result of one or more instance-manipulations applied to the current instance.\"",
    },
    "300" : {
        "phrase": "Multiple Choices",
        "description": "\"indicates that the target resource has more than one representation, each with its own more specific identifier, and information about the alternatives is being provided so that the user (or user agent) can select a preferred representation by redirecting its request to one or more of those identifiers.\"",
    },
    "301" : {
        "phrase": "Moved Permanently",
        "description": "\"indicates that the target resource has been assigned a new permanent URI and any future references to this resource ought to use one of the enclosed URIs.\"",
    },
    "302" : {
        "phrase": "Found",
        "description": "\"indicates that the target resource resides temporarily under a different URI.\"",
    },
    "303" : {
        "phrase": "See Other",
        "description": "\"indicates that the server is redirecting the user agent to a different resource, as indicated by a URI in the Location header field, that is intended to provide an indirect response to the original request.\"",
    },
    "304" : {
        "phrase": "Not Modified",
        "description": "\"indicates that a conditional GET request has been received and would have resulted in a 200 (OK) response if it were not for the fact that the condition has evaluated to false.\"",
    },
    "305" : {
        "phrase": "Use Proxy",
        "description": "*deprecated*",
    },
    "307" : {
        "phrase": "Temporary Redirect",
        "description": "\"indicates that the target resource resides temporarily under a different URI and the user agent MUST NOT change the request method if it performs an automatic redirection to that URI.\"",
    },
    "308" : {
        "phrase": "Permanent Redirect",
        "description": "\"The target resource has been assigned a new permanent URI and any future references to this resource outght to use one of the enclosed URIs. [...] This status code is similar to 301 Moved Permanently (Section 7.3.2 of rfc7231), except that it does not allow rewriting the request method from POST to GET.\"",
    },
    "400" : {
        "phrase": "Bad Request",
        "description": "\"indicates that the server cannot or will not process the request because the received syntax is invalid, nonsensical, or exceeds some limitation on what the server is willing to process.\"",
    },
    "401" : {
        "phrase": "Unauthorized",
        "description": "\"indicates that the request has not been applied because it lacks valid authentication credentials for the target resource.\"",
    },
    "402" : {
        "phrase": "Payment Required",
        "description": "*reserved*",
    },
    "403" : {
        "phrase": "Forbidden",
        "description": "\"indicates that the server understood the request but refuses to authorize it.\"",
    },
    "404" : {
        "phrase": "Not Found",
        "description": "\"indicates that the origin server did not find a current representation for the target resource or is not willing to disclose that one exists.\"",
    },
    "405" : {
        "phrase": "Method Not Allowed",
        "description": "\"indicates that the method specified in the request-line is known by the origin server but not supported by the target resource.\"",
    },
    "406" : {
        "phrase": "Not Acceptable",
        "description": "\"indicates that the target resource does not have a current representation that would be acceptable to the user agent, according to the proactive negotiation header fields received in the request, and the server is unwilling to supply a default representation.\"",
    },
    "407" : {
        "phrase": "Proxy Authentication Required",
        "description": "\"is similar to 401 (Unauthorized), but indicates that the client needs to authenticate itself in order to use a proxy.\"",
    },
    "408" : {
        "phrase": "Request Timeout",
        "description": "\"indicates that the server did not receive a complete request message within the time that it was prepared to wait.\"",
    },
    "409" : {
        "phrase": "Conflict",
        "description": "\"indicates that the request could not be completed due to a conflict with the current state of the resource.\"",
    },
    "410" : {
        "phrase": "Gone",
        "description": "\"indicates that access to the target resource is no longer available at the origin server and that this condition is likely to be permanent.\"",
    },
    "411" : {
        "phrase": "Length Required",
        "description": "\"indicates that the server refuses to accept the request without a defined Content-Length.\"",
    },
    "412" : {
        "phrase": "Precondition Failed",
        "description": "\"indicates that one or more preconditions given in the request header fields evaluated to false when tested on the server.\"",
    },
    "413" : {
        "phrase": "Payload Too Large",
        "description": "\"indicates that the server is refusing to process a request because the request payload is larger than the server is willing or able to process.\"",
    },
    "414" : {
        "phrase": "URI Too Long",
        "description": "\"indicates that the server is refusing to service the request because the request-target is longer than the server is willing to interpret.\"",
    },
    "415" : {
        "phrase": "Unsupported Media Type",
        "description": "\"indicates that the origin server is refusing to service the request because the payload is in a format not supported by the target resource for this method.\"",
    },
    "416" : {
        "phrase": "Range Not Satisfiable",
        "description": "\"indicates that none of the ranges in the request's Range header field overlap the current extent of the selected resource or that the set of ranges requested has been rejected due to invalid ranges or an excessive request of small or overlapping ranges.\"",
    },
    "417" : {
        "phrase": "Expectation Failed",
        "description": "\"indicates that the expectation given in the request's Expect header field could not be met by at least one of the inbound servers.\"",
    },
    "418" : {
        "phrase": "I'm a teapot",
        "description": "\"Any attempt to brew coffee with a teapot should result in the error code 418 I'm a teapot.\"",
    },
    "426" : {
        "phrase": "Upgrade Required",
        "description": "\"indicates that the server refuses to perform the request using the current protocol but might be willing to do so after the client upgrades to a different protocol.\"",
    },
    "422" : {
        "phrase": "Unprocessable Entity",
        "description": "\"means the server understands the content type of the request entity (hence a 415(Unsupported Media Type) status code is inappropriate), and the syntax of the request entity is correct (thus a 400 (Bad Request) status code is inappropriate) but was unable to process the contained instructions.\"",
    },
    "423" : {
        "phrase": "Locked",
        "description": "\"means the source or destination resource of a method is locked.\"",
    },
    "424" : {
        "phrase": "Failed Dependency",
        "description": "\"means that the method could not be performed on the resource because the requested action depended on another action and that action failed.\"",
    },
    "428" : {
        "phrase": "Precondition Required",
        "description": "\"indicates that the origin server requires the request to be conditional.\"",
    },
    "429" : {
        "phrase": "Too Many Requests",
        "description": "\"indicates that the user has sent too many requests in a given amount of time (\"rate limiting\").\"",
    },
    "431" : {
        "phrase": "Request Header Fields Too Large",
        "description": "\"indicates that the server is unwilling to process the request because its header fields are too large.\"",
    },
    "451" : {
        "phrase": "Unavailable For Legal Reasons",
        "description": "\"This status code indicates that the server is denying access to the resource in response to a legal demand.\"",
    },
    "500" : {
        "phrase": "Internal Server Error",
        "description": "\"indicates that the server encountered an unexpected condition that prevented it from fulfilling the request.\"",
    },
    "501" : {
        "phrase": "Not Implemented",
        "description": "\"indicates that the server does not support the functionality required to fulfill the request.\"",
    },
    "502" : {
        "phrase": "Bad Gateway",
        "description": "\"indicates that the server, while acting as a gateway or proxy, received an invalid response from an inbound server it accessed while attempting to fulfill the request.\"",
    },
    "503" : {
        "phrase": "Service Unavailable",
        "description": "\"indicates that the server is currently unable to handle the request due to a temporary overload or scheduled maintenance, which will likely be alleviated after some delay.\"",
    },
    "504" : {
        "phrase": "Gateway Time-out",
        "description": "\"indicates that the server, while acting as a gateway or proxy, did not receive a timely response from an upstream server it needed to access in order to complete the request.\"",
    },
    "505" : {
        "phrase": "HTTP Version Not Supported",
        "description": "\"indicates that the server does not support, or refuses to support, the protocol version that was used in the request message.\"",
    },
    "506" : {
        "phrase": "Variant Also Negotiates",
        "description": "\"indicates that the server has an internal configuration error: the chosen variant resource is configured to engage in transparent content negotiation itself, and is therefore not a proper end point in the negotiation process.\"",
    },
    "507" : {
        "phrase": "Insufficient Storage",
        "description": "\"means the method could not be performed on the resource because the server is unable to store the representation needed to successfully complete the request.\"",
    },
    "511" : {
        "phrase": "Network Authentication Required",
        "description": "\"indicates that the client needs to authenticate to gain network access.\"",
    }
}


# LOG para todas las funciones
__name__ = "wsgi"
sys.stdout = sys.stderr
if DEBUG:
    logPath = ''
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)
    hdlr = logging.FileHandler(logPath+__name__+'.log')
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    hdlr.setFormatter(formatter)
    LOG.addHandler(hdlr)
    LOG.info("wsgi modo local INICIO")
else:
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)

class Pywsgi:
    """Gestion de entorno Web Server Gateway Interface"""

    __name__ = "pywsgi"
    sys.stdout = sys.stderr

    if DEBUG:
        logPath = ''
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)
        hdlr = logging.FileHandler(logPath+__name__+'.log')
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        hdlr.setFormatter(formatter)
        LOG.addHandler(hdlr)
        LOG.info("Session modo local INICIO")
    else:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)

    def __init__(self, environment, start_response, **kwargs):
        try:
            self.request_body_size = 0
            self._p = None
            self._q = None
            self.parametros = None
            self.ajax = False
            self.environ = environment
            self.start_response = start_response
            # comprobar ajax
            if self.environ.get('HTTP_X_REQUESTED_WITH','').lower() == 'xmlhttprequest':
                self.ajax_request = True
            # Comprobar si REQUEST_METHOD esta permitido
            if self.environ.get('REQUEST_METHOD','').upper() not in ['GET','POST','PUT','DELETE']:
                self.start_response(getStatus('405'),
                 [('content-type', 'text/plain'),
                  ('Cache-Control', 'no-store'),
                  ('Content-Language','es')])
                #return ('Method Not Allowed',)
            else:
                    #*********************************************************************************************************************************
                    # La variable de environ CONTENT_LENGTH puede ser 0 o estar ausente en las variables de environ
                    # esta relacionada con el contenido de wsgi.input
                    try:
                        request_body_size = int(self.environ.get('CONTENT_LENGTH','0'))
                    except Exception as e:
                        request_body_size = 0
                    if request_body_size != 0:
                        lines = []
                        for line in environ['wsgi.input']:
                            lines.append(line)
                        self._p =  ''.join(lines)
                    else:
                        #*********************************************************************************************************************************
                        # QUERY_STRING puede no tener contenido o estar ausente en las variables de environ
                        #
                        try:
                            if self.environ.get('QUERY_STRING',None):
                                """ QUERY_STRING The portion of the request URL that follows the "?" , if any. May be empty or absent."""
                                self._p = environ.get('QUERY_STRING','')
                        except Exception as e:
                            pass
                    if self._p is not None:
                        if self.environ.get('CONTENT-TRANSFER-ENCODING','').upper()=='BASE64':
                            # comprobar si parametro recibido esta codificado en base64
                            self._p = base64.b64decode(parse_qs(unicode(self._p).encode('utf-8'))['base64'].replace("%3D","=").replace("%2F","/").replace("%2B","+"))
                        # obtener parametros
                        if CONTENT_TYPE.get(self.environ.get('CONTENT_TYPE','').split(';')[0],None):
                            eval(self.environ.get('CONTENT_TYPE','').split(';')[0])
                    else:
                        #self.parametros = json.dumps({"request":0})
                        self.parametros = None
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error("Error en metodo '__init__' <%s> en linea %s !!!\n" % (str(e),tb.tb_lineno))

    def get_env(self, key):
        return self.environment.get(key)

    def get_request(self):
        return self.parametros

    def request(self):
        if re.search('^\{.*}$', self._p ):
            # comprobar si parametro recibido contiene un formato JSON
            json_q = json.loads(self._p)
            for n in json_q.keys():
                json_q[n] = latin2html(json_q[n])
            self.parametros = json.dumps(json_q)
        else:
            # formato recibido en parametro es una cadena de clave=valor
            json_q = parse_qs(self._p)
            for n in json_q.keys():
                json_q[n] = latin2html(json_q[n])
            self.parametros = json.dumps(json_q)

    def request_json(self):
        ''' tratamiento de parametros content-type <application/json>'''
        if re.search('^\{.*}$', self._p ):
            # comprobar si parametro recibido contiene un formato JSON
            json_q = json.loads(self._p)
            LOG.debug('parametro procesado application/json %s' % json.dumps(json_q))
            for n in json_q.keys():
                LOG.debug('parametro procesado application/json %s' % json_q[n])
                if type(json_q[n]).__name__ == 'list':
                    for n1 in json_q[n]:
                        if type(n1).__name__ == 'dict':
                            for n2 in n1.keys():
                                n1[n2] = latin2html(n1[n2])
                else:
                    json_q[n] = latin2html(json_q[n])

            self.parametros = json.dumps(json_q)

    def request_multipart(self):
        ''' tratamiento de parametros content-type <multipart/form-data>'''
        _q_list = self._p.split('\r')
        json_q = dict()
        # separar contenido por 'boundary'
        _q_list = self._p.split(_q_list[0])
        _pp = dict()
        for n in range(len(_q_list)):
            json_q[n] = _q_list[n].split('\r\n')
            if len(json_q[n])>3:
                if json_q[n][2].lower().startswith('content-type'):
                    content_disposition_name = None
                    content_disposition_filename = None
                    # reconstruir list como fichero
                    json_q[n][4] += '\r\n'
                    for m in range(5,len(json_q[n])-1):
                        json_q[n][4] += json_q[n][m]+'\r\n'
                    # Content-Disposition: form-data; name="file"; filename="12816.jpg"
                    # clave
                    content_disposition_name = json_q[n][1].split(':')[1].split(';')[1].split('=')[1].replace('"', '')
                    # filename
                    content_disposition_filename = json_q[n][1].split(':')[1].split(';')[2].split('=')[1].replace('"', '')
                    if content_disposition_name is not None and content_disposition_filename is not None:
                        # si el form permite multiples ficheros, los nombres se almacenan como list
                        if _pp.get(content_disposition_name,None):
                            _pp[content_disposition_name].append(content_disposition_filename)
                        else:
                            _pp[content_disposition_name]= [content_disposition_filename]
                    # guardar el fichero
                    if len(json_q[n][4])>0:
                        outfile = open(upload_dir+'/'+content_disposition_filename, 'wb')
                        outfile.write(json_q[n][4])
                        outfile.close()
                else:
                    # es un valor
                    content_disposition_name = None
                    content_type = None
                    for m in range(len(json_q[n])):
                        if json_q[n][m].lower().startswith('content-disposition'):
                            # clave
                            content_disposition_name = json_q[n][1].split(':')[1].split(';')[1].split('=')[1].replace('"', '')
                        elif len(json_q[n][m])>0:
                            # valor
                            content_type = latin2html(json_q[n][3].strip())
                    if content_disposition_name is not None and content_type is not None:
                        _pp[content_disposition_name]=content_type
        # liberar memoria
        _q_list = None
        json_q = None
        #
        self.parametros = json.dumps(_pp)

def getStatus(status_id):
    #Tratamiento de errores en LOG
    __name__ = "wsgi.getStatus"
    sys.stdout = sys.stderr

    if DEBUG:
        logPath = ''
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)
        hdlr = logging.FileHandler(logPath+__name__+'.log')
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        hdlr.setFormatter(formatter)
        LOG.addHandler(hdlr)
        LOG.info("Session modo local INICIO")
    else:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)
    try:
        if STATUS_CODE.get(status_id,None):
            return "{0} {1}".format(status_id,STATUS_CODE[status_id]['phrase'])
        else:
            status_id = "200"
            return "{0} {1}".format(status_id,STATUS_CODE[status_id]['phrase'])
    except Exception as e:
        tb = sys.exc_info()[2]
        LOG.debug("Error <%s> en linea %s !!!\n" % (str(e),tb.tb_lineno))


_hexdig = '0123456789ABCDEFabcdef'
_hextochr = dict((a+b, chr(int(a+b,16)))
                 for a in _hexdig for b in _hexdig)

def unquote(s):
    """unquote('abc%20def') -> 'abc def'."""
    res = s.split('%')
    # fastpath
    if len(res) == 1:
        return s
    s = res[0]
    for item in res[1:]:
        try:
            s += _hextochr[item[:2]] + item[2:]
        except KeyError:
            s += '%' + item
        except UnicodeDecodeError:
            s += unichr(int(item[:2], 16)) + item[2:]
    return s

def parse_qs(qs, keep_blank_values=0, strict_parsing=0):
    """Parse a query given as a string argument.

        Arguments:

        qs: percent-encoded query string to be parsed

        keep_blank_values: flag indicating whether blank values in
            percent-encoded queries should be treated as blank strings.
            A true value indicates that blanks should be retained as
            blank strings.  The default false value indicates that
            blank values are to be ignored and treated as if they were
            not included.

        strict_parsing: flag indicating what to do with parsing errors.
            If false (the default), errors are silently ignored.
            If true, errors raise a ValueError exception.
    """
    __name__ = "parse_qs"
    sys.stdout = sys.stderr
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
    LOG = logging.getLogger(__name__)

    data_dict = {}
    for name, value in parse_qsl(qs, keep_blank_values, strict_parsing):
        if data_dict.get(name,None):
            if DEBUG:
                LOG.debug('qs parametros %s %s' % (name,value))
            data_dict[name].append(string2unicode(value))
        else:
            if DEBUG:
                LOG.debug('qs parametros %s %s' % (name,value))
            data_dict[name] = string2unicode(value)
    return data_dict

def parse_qsl(qs, keep_blank_values=0, strict_parsing=0):
    """Parse a query given as a string argument.

    Arguments:

    qs: percent-encoded query string to be parsed

    keep_blank_values: flag indicating whether blank values in
        percent-encoded queries should be treated as blank strings.  A
        true value indicates that blanks should be retained as blank
        strings.  The default false value indicates that blank values
        are to be ignored and treated as if they were  not included.

    strict_parsing: flag indicating what to do with parsing errors. If
        false (the default), errors are silently ignored. If true,
        errors raise a ValueError exception.

    Returns a list, as G-d intended.
    """
    pairs = [s2 for s1 in qs.split('&') for s2 in s1.split(';')]
    r = []
    for name_value in pairs:
        if not name_value and not strict_parsing:
            continue
        nv = name_value.split('=', 1)
        if len(nv) != 2:
            if strict_parsing:
                raise ValueError("bad query field: %r" % name_value)
            # Handle case of a control-name with no equal sign
            if keep_blank_values:
                nv.append('')
            else:
                continue
        if len(nv[1]) or keep_blank_values:
            name = unquote(nv[0].replace('+', ' '))
            value = unquote(nv[1].replace('+', ' '))
            r.append((name, string2unicode(value)))
    return r

def parse_qj(qs, keep_blank_values=0, strict_parsing=0):
    """Parse a query given as a string argument.

        Arguments:

        qs: percent-encoded query string to be parsed

        keep_blank_values: flag indicating whether blank values in
            percent-encoded queries should be treated as blank strings.
            A true value indicates that blanks should be retained as
            blank strings.  The default false value indicates that
            blank values are to be ignored and treated as if they were
            not included.

        strict_parsing: flag indicating what to do with parsing errors.
            If false (the default), errors are silently ignored.
            If true, errors raise a ValueError exception.
    """
    data_dict = {}
    for name, value in parse_qjl(qs, keep_blank_values, strict_parsing):
        if data_dict.get(name,None):
            data_dict[name].append(string2unicode(value))
        else:
            data_dict[name] = string2unicode(value)
    return data_dict

def parse_qjl(qs, keep_blank_values=0, strict_parsing=0):
    """Parse a query given as a string JSON argument.

    Arguments:

    qs: percent-encoded query string to be parsed

    keep_blank_values: flag indicating whether blank values in
        percent-encoded queries should be treated as blank strings.  A
        true value indicates that blanks should be retained as blank
        strings.  The default false value indicates that blank values
        are to be ignored and treated as if they were  not included.

    strict_parsing: flag indicating what to do with parsing errors. If
        false (the default), errors are silently ignored. If true,
        errors raise a ValueError exception.

    Returns a list, as G-d intended.
    """
    if qs[0:1]=='{' and qs[-1:]=='}':
        qs = qs[1:-1]
    pairs = [s2 for s1 in qs.split(',') for s2 in s1.split(';')]
    r = []
    for name_value in pairs:
        if not name_value and not strict_parsing:
            continue
        nv = name_value.split(':', 1)
        if len(nv) != 2:
            if strict_parsing:
                raise ValueError("bad query field: %r" % name_value)
            # Handle case of a control-name with no equal sign
            if keep_blank_values:
                nv.append('')
            else:
                continue
        if len(nv[1]) or keep_blank_values:
            name = unquote(nv[0].replace('+', ' '))
            name = unquote(name.replace('"', ''))
            value = unquote(nv[1].replace('+', ' '))
            value = unquote(value.replace('"', ''))
            r.append((name, string2unicode(value)))
    return r

if __name__ == "__main__":
    if len(sys.argv) < 2:
        sys.stderr.write("for help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (u'''
        Gestion de wsgi''')
    print getStatus('200')

